/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-2004 CONDOR Team, Computer Sciences Department,
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.
 * No use of the CONDOR Software Program Source Code is authorized
 * without the express consent of the CONDOR Team.  For more information
 * contact: CONDOR Team, Attention: Professor Miron Livny,
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685,
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure
 * by the U.S. Government is subject to restrictions as set forth in
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison,
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

// Declaration of ClassAdList class.

#ifndef _ClassAdList_h_
#define _ClassAdList_h_

#include <sys/types.h>
#include <string>
#include <map>

#include "classad_distribution.h"
#include "ErrorStatus.h"

typedef multimap<string, ClassAd *>::iterator ClassAdListIterator;
typedef multimap<string, ClassAd *>::const_iterator ConstClassAdListIterator;

class ClassAdList {
public:
	// -------------------------------------
	// Constructors and destructors.
	// -------------------------------------
	/**
	 * Constructor.
	 */
	ClassAdList();

	/**
	 * Destructor -- deletes any classads still in the list.
	 */
	~ClassAdList();

	// -------------------------------------
	// Accessors.
	// -------------------------------------
	/**
	 * Insert the given ad at the end of the list.
	 * @param The ad to insert.
	 */
	void InsertAd(ClassAd *ad);

	/* 
	 * Insert the given ad at the end of the list.
	 * @param The name of the ad.
	 * @param The ad to insert.
	 */
	void InsertAd(const string &name, ClassAd *ad);

	/**
	 * Get the size (number of ads) of this object.
	 * @return The number of ads in this object.
	 */
	int Size() const { return classAds.size(); }

	/**
	 * Begin iterator.
	 */
	ClassAdListIterator begin() { return classAds.begin(); }

	/**
	 * Begin const iterator.
	 */
	ConstClassAdListIterator begin() const { return classAds.begin(); }

	/**
	 * End iterator.
	 */
	ClassAdListIterator end() { return classAds.end(); }

	/**
	 * End const iterator.
	 */
	ConstClassAdListIterator end() const { return classAds.end(); }

	/**
	 * Find the ad (if any) with the given name.  Note that if there is
	 * more than one ad with the same name, a warning is generated.
	 * @param The name of the ad to find.
	 * @param Reference to pointer that will point to the ad.
	 */
	ErrorStatus FindAd(const string &name, ClassAd *&ad);

	// -------------------------------------
	// I/O.
	// -------------------------------------
	/**
	 * Read ads from a file into this object.
	 * @param The name of the file ("-" for stdin).
	 * @param Whether the file is XML.
	 */
	ErrorStatus ReadFromFile(const string &fileName, bool isXml);

	/**
	 * Write ads from this object into a file.
	 * @param The name of the file ("-" for stdout).
	 * @param Whether the file is XML.
	 */
	ErrorStatus WriteToFile(const string &fileName, bool isXml);

	// -------------------------------------
	// Misc.
	// -------------------------------------

	/**
	 * Move ads from the given list to this list (removes all ads from
	 * the given list).
	 * @param The list from which to move the ads.
	 */
	void MoveAds(ClassAdList &list);

	/**
	 * Evaluate an attribute that is a list of ClassAds, and place copies
	 * of those ads in this list.
	 * @param The ad containing the relevant attribute.
	 * @param The attribute name.
	 */
	ErrorStatus ReadFromAttr(const ClassAd *ad, const string& attrName);

private:
	/**
	 * Read ads from an istream into this object.
	 * @param The istream.
	 * @param Whether the data is XML.
	 */
	ErrorStatus ReadFromStream(istream &istr, bool isXml);

	/**
	 * Write ads from this object to an ostream.
	 * @param The ostream.
	 * @param Whether the data is XML.
	 */
	ErrorStatus WriteToStream(ostream &istr, bool isXml);

	// Note: this *must* be a multimap, not a map, or we'll lose stuff.
	multimap<string, ClassAd *>		classAds;
};

#endif /* _ClassAdList_h_ */

/*============================================================================*/
