/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-2004 CONDOR Team, Computer Sciences Department,
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.
 * No use of the CONDOR Software Program Source Code is authorized
 * without the express consent of the CONDOR Team.  For more information
 * contact: CONDOR Team, Attention: Professor Miron Livny,
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685,
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure
 * by the U.S. Government is subject to restrictions as set forth in
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison,
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#ifndef _Output_h_
#define _Output_h_

#include <string>

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#define OUTPUT_OUT(msg) Output::GetInstance()->MsgOut( \
		Output::VERBOSITY_OUTPUT, msg)

#define WARN_OUT(msg) Output::GetInstance()->MsgOut( \
		Output::VERBOSITY_WARN, msg, __FILE__, __LINE__)

#define ERROR_OUT(msg) Output::GetInstance()->MsgOut( \
		Output::VERBOSITY_ERROR, msg, __FILE__, __LINE__)

#define DIAG1_OUT(msg) Output::GetInstance()->MsgOut( \
		(DEBUG >= 1) ? Output::VERBOSITY_OUTPUT : \
		Output::VERBOSITY_DIAG1, msg, __FILE__, __LINE__)

// Like DIAG1_OUT, except no source code info.
#define DIAG1_OUT_NS(msg) Output::GetInstance()->MsgOut( \
		(DEBUG >= 1) ? Output::VERBOSITY_OUTPUT : \
		Output::VERBOSITY_DIAG1, msg)

#define DIAG2_OUT(msg) Output::GetInstance()->MsgOut( \
		(DEBUG >= 2) ? Output::VERBOSITY_OUTPUT : \
		Output::VERBOSITY_DIAG2, msg, __FILE__, __LINE__)

class Output {
public:
	enum Verbosity {
		VERBOSITY_NONE = 0,
		VERBOSITY_OUTPUT,
		VERBOSITY_ERROR,
		VERBOSITY_WARN,
		VERBOSITY_DIAG1,
		VERBOSITY_DIAG2,
	};
	static const Verbosity VERBOSITY_DEFAULT = VERBOSITY_WARN;

	/**
	 * Get an instance of an Output object (constructing one if necessary).
	 * @return A pointer to an Output object.
	 */
	static Output *GetInstance();

	/**
	 * Constructor.
	 */
	Output() : verbosity(VERBOSITY_DEFAULT) {}

	/**
	 * Set the verbosity level.
	 * @param The new verbosity level.
	 */
	void SetVerbosity(int verbosity) { this->verbosity = verbosity; }

	/**
	 * Output a message (if the verbosity level is appropriate).
	 * @param The type of the message.
	 * @param The message string.
	 * @param The message source file.
	 * @param The message source line.
	 */
	void MsgOut(Verbosity msgType, string msg,
			string srcFile = "", int srcLine = -1) const;

private:
	static Output *	instance;
	int			verbosity;
};

#endif /* _Output_h_ */
