/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-2004 CONDOR Team, Computer Sciences Department,
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.
 * No use of the CONDOR Software Program Source Code is authorized
 * without the express consent of the CONDOR Team.  For more information
 * contact: CONDOR Team, Attention: Professor Miron Livny,
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685,
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure
 * by the U.S. Government is subject to restrictions as set forth in
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison,
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

// Declaration of ProblemProcess class.  This class implements the
// processing to generate problem ads.

#ifndef _ProblemProcess_h_
#define _ProblemProcess_h_

#include "ErrorStatus.h"
#include "ClassAdList.h"
#include "SendMail.h"

// Note: arguments marked with a const comment do not "really" change the
// given object, but can't be truly declared const because they use
// iterators, etc.

class ProblemProcess
{
public:
	/**
	 * Default constructor.
	 */
	ProblemProcess::ProblemProcess();

	/**
	 * Destructor.
	 */
	ProblemProcess::~ProblemProcess();

	/**
	 * Take state ads and trigger ads, generate any action ads that
	 * are appropriate.
	 * @param List of state ads.
	 * @param List of trigger ads.
	 * @param List of action ads (should be empty).
	 * @return ErrorStatus object giving the status of the operation.
	 */
	ErrorStatus GenerateActionAds(/* const */ ClassAdList &stateAds,
			/* const */ ClassAdList &triggerAds, ClassAdList &actionAds);

	/**
	 * Process the given action ads -- send appropriate email or whatever.
	 * @param List of action ads.
	 */
	ErrorStatus ProcessActionAds(/* const */ClassAdList &actionAds);

	/**
	 * Get the error status of this object.
	 * @return ErrorStatus of this object.
	 */
	const ErrorStatus &ObjectStatus() const { return objectStatus; }

private:
	/**
	 * Check whether the given state matches the given trigger; if so,
	 * insert a corresponding action ad into the given action ads list.
	 * @param State ClassAd.
	 * @param Trigger ClassAd.
	 * @param List of action ads.
	 * @return ErrorStatus object giving the status of the operation.
	 */
	ErrorStatus CheckAction(/* const */ ClassAd *state,
			/* const */ ClassAd *trigger, ClassAdList &actionAds);

	/**
	 * Flatten trigger.ad in the match ad, and replace the original
	 * expression with the flattened one.
	 * @return ErrorStatus object giving the status of the operation.
	 */
	ErrorStatus FlattenTriggerAd();

	/**
	 * Evaluate the appropriate action attributes and add a classad to
	 * the action ads.
	 * @param List of action ads.
	 * @return ErrorStatus object giving the status of the operation.
	 */
	ErrorStatus EvalActionAttrs(ClassAdList &actionAds);

	ErrorStatus	objectStatus;
	ClassAd 	*matchAd;
	ClassAd		*matchStateAd;
	ClassAd		*matchTriggerAd;
	SendMail	sm;
};

#endif /* _ProblemProcess_h_ */

/*============================================================================*/
