/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-2004 CONDOR Team, Computer Sciences Department,
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.
 * No use of the CONDOR Software Program Source Code is authorized
 * without the express consent of the CONDOR Team.  For more information
 * contact: CONDOR Team, Attention: Professor Miron Livny,
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685,
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure
 * by the U.S. Government is subject to restrictions as set forth in
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison,
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

// Implementation of SendMail class.

#include "trigger_common.h"

#include <iostream>
#include <sys/wait.h>
#include <unistd.h>
#include <strstream>
#include <errno.h>

#include "SendMail.h"
#include "classad_triggers.h"

static const int DEBUG = 0;

//-------------------------------------------------------------------------
SendMail::SendMail()
{
	const char *	userName = getenv("LOGNAME");
	if ( userName == NULL ) {
		userName = cuserid(NULL);
	}

	currentUser = userName;
}

//-------------------------------------------------------------------------
SendMail::~SendMail()
{
	DIAG1_OUT("SendMail::~SendMail()");

	map<string, MailFile *>::iterator	mapI;
	for ( mapI = mailList.begin(); mapI != mailList.end(); ++mapI ) {
		MailFile	*mf = mapI->second;
		delete mf;
	}
}

//-------------------------------------------------------------------------
ErrorStatus
SendMail::CreateActionMail(const string &to, const string &subject,
		const string &text)
{
	DIAG1_OUT("SendMail::CreateActionMail(" + to + ", " + subject + ", " +
			text + ")");

	ErrorStatus		result;

		// We should probably change from hard-coded values to values defined
		// in a classad...
	string realTo = to;
	if ( realTo == "root" ) {
		realTo = "wenger";
		realTo = "current user";
	}

	if ( realTo == "current user" ) {
		realTo = currentUser;
	}

		// Find or create a MailFile for the given email address.
	MailFile	*mf = NULL;

	map<string, MailFile *>::iterator	mapI;
	mapI = mailList.find(realTo);
	if ( mapI == mailList.end() ) {
		mf = new MailFile();
		result += mf->ObjectStatus();
		mailList[realTo] = mf;
		mf->SetSubject(subject);
	} else {
		mf = mapI->second;
		mf->SetSubject("multiple triggers");
	}

		// Append the given message.
	result += mf->AppendText(text);

	return result;
}

//-------------------------------------------------------------------------
ErrorStatus
SendMail::SendAllMails()
{
	DIAG1_OUT("SendMail::SendAllMails()");

	ErrorStatus		result;

	map<string, MailFile *>::iterator	mapI;
	for ( mapI = mailList.begin(); mapI != mailList.end(); ++mapI ) {
		string		to = mapI->first;
		MailFile	*mf = mapI->second;
		if ( mf->ObjectStatus().IsComplete() ) {
			mf->Flush();
			SendActionMail(to, mf->GetSubject(), mf->GetFileName());
		}
	}

	return result;
}

//-------------------------------------------------------------------------
ErrorStatus
SendMail::SendActionMail(const string &to, const string &subject,
		const string &fileName)
{
	DIAG1_OUT("SendMail::SendActionMail(" + to + ", " + subject +
			", " + fileName + ")");

	ErrorStatus		result;

	pid_t		pid = fork();
	if ( pid == 0 ) {
			// Child.
		execlp("./send_action_email", "./send_problem_email", to.c_str(),
				subject.c_str(), fileName.c_str(), NULL);

			// We only get here if exec failed...
		ostrstream		msg;
		msg << "Exec of mhmail failed with errno " << errno << ends;
		ERROR_OUT(string(msg.str()));
		exit(1);

	} else if ( pid == -1 ) {
			// Parent, error.
		ostrstream		msg;
		msg << "Fork failed with errno " << errno << ends;
		ERROR_OUT(string(msg.str()));
		result += ErrorStatus(StatusFailed, ERROR_SYSCALL);

	} else {
			// Parent, no error.
		int		status;
		if ( wait(&status) == -1 ) {
			ostrstream		msg;
			msg << "Wait failed with errno " << errno << ends;
			ERROR_OUT(string(msg.str()));
			result += ErrorStatus(StatusFailed, ERROR_SYSCALL);
			
		} else {
			if ( WIFEXITED(status) ) {
				if ( WEXITSTATUS(status) != 0 ) {
					ostrstream		msg;
					msg << "Child exited with status " <<
							WEXITSTATUS(status) << ends;
					ERROR_OUT(string(msg.str()));
					result += ErrorStatus(StatusFailed, ERROR_CHILD);
				}
			} else {
				ERROR_OUT("Child process exited abnormally");
				result += ErrorStatus(StatusFailed, ERROR_CHILD);
			}
		}
	}

	return result;
}

//-------------------------------------------------------------------------
SendMail::MailFile::MailFile() : fileName(""), os(NULL)
{
	DIAG2_OUT("SendMail::MailFile::MailFile()");

	char	*tmpName = strdup("/tmp/MailFileXXXXXX");
	int		fd = mkstemp(tmpName);
	if ( fd != -1 ) {
		close(fd);
		fileName = tmpName;
		os = new ofstream(tmpName);
	} else {
		objectStatus += ErrorStatus(StatusFailed, ERROR_FILE_IO);
		ERROR_OUT(string("Could not create temp mail file ") + tmpName);
	}

	free(tmpName);
}

//-------------------------------------------------------------------------
SendMail::MailFile::~MailFile()
{
	DIAG2_OUT("SendMail::MailFile::~MailFile()");

	if ( os ) {
		os->close();
		delete os;
	}

	if ( fileName != "" ) {
		if ( unlink(fileName.c_str()) != 0 ) {
			ERROR_OUT("Unable to unlink temporary file " + fileName);
		}
	}
}

//-------------------------------------------------------------------------
ErrorStatus
SendMail::MailFile::AppendText(const string &text)
{
	ErrorStatus		result;

	if ( os ) {
		*os << text << endl << endl;
	} else {
		ERROR_OUT("Invalid output stream");
		result = ErrorStatus(StatusFailed, ERROR_INTERNAL_STATE);
	}

	return result;
}

/*============================================================================*/
