/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-2004 CONDOR Team, Computer Sciences Department,
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.
 * No use of the CONDOR Software Program Source Code is authorized
 * without the express consent of the CONDOR Team.  For more information
 * contact: CONDOR Team, Attention: Professor Miron Livny,
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685,
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure
 * by the U.S. Government is subject to restrictions as set forth in
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison,
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

// Declaration of SendMail class.  This class is used to actually send
// problem emails.

#ifndef _SendMail_h_
#define _SendMail_h_

#include <iostream>
#include <fstream>
#include <map>

#include "ErrorStatus.h"

class SendMail
{
public:
	// -------------------------------------
	// Constructors and destructors.
	// -------------------------------------

	/**
	 * Default constructor.
	 */
	SendMail();

	/**
	 * Destructor.
	 */
	~SendMail();

	// -------------------------------------
	// Misc. methods.
	// -------------------------------------

	/**
	 * Add a message to the list of problem emails we're going to send.
	 * @param The email address to send mail to.
	 * @param The subject of the message.
	 * @param The text to send as the body of the message.
	 * @return ErrorStatus object giving the status of the operation.
	 */
	ErrorStatus CreateActionMail(const string &to, const string &subject,
			const string &text);

	/**
	 * Send all of the accumulated emails.
	 * @return ErrorStatus object giving the status of the operation.
	 */
	ErrorStatus SendAllMails();

private:

	class MailFile {
	public:
		/**
		 * Default constructor.
		 */
		MailFile();

		/**
		 * Get the error status of this object.
		 * @return ErrorStatus of this object.
		 */
		const ErrorStatus &ObjectStatus() const { return objectStatus; }

		/**
		 * Destructor.
		 */
		~MailFile();

		/**
		 * Append the given text to the email for this object's email
		 * address.
		 * @param The text to append.
		 * @return ErrorStatus object giving the status of the operation.
		 */
		ErrorStatus AppendText(const string &text);

		/**
		 * Get the file name of this object's temporary mail file.
		 * @return The file name of this object's temporary mail file.
		 */
		const string &GetFileName() const { return fileName; }

		/**
		 * Set the subject for this email.
		 * @param The subject that the email should have.
		 */
		void SetSubject(const string &subject) { this->subject = subject; }

		/**
		 * Get the subject for this email.
		 * @return The subject of this email.
		 */
		const string &GetSubject() const { return subject; }


		/**
		 * Flush this object's ostream.
		 */
		void Flush() { os->flush(); }

	private:
		ErrorStatus	objectStatus;
		string		fileName;
		ofstream	*os;
		string		subject;
	};

	/**
	 * Send problem email.
	 * @param The email address to send mail to.
	 * @param The name of the file containing the text to send as the
	 * body of the message.
	 * @param The subject of the message.
	 * @return ErrorStatus object giving the status of the operation.
	 */
	ErrorStatus SendActionMail(const string &to, const string &subject,
			const string &fileName);

	map<string, MailFile *>	mailList;

	string		currentUser;
};

#endif /* _SendMail_h_ */

/*============================================================================*/
