/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-2004 CONDOR Team, Computer Sciences Department,
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.
 * No use of the CONDOR Software Program Source Code is authorized
 * without the express consent of the CONDOR Team.  For more information
 * contact: CONDOR Team, Attention: Professor Miron Livny,
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685,
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure
 * by the U.S. Government is subject to restrictions as set forth in
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison,
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

// Declaration of StateProcess class.  This class implements the
// processing to generate state ads.

#ifndef _StateProcess_h_
#define _StateProcess_h_

#include "ErrorStatus.h"
#include "ClassAdList.h"

// Note: arguments marked with a const comment do not "really" change the
// given object, but can't be truly declared const because they use
// iterators, etc.

class StateProcess
{
public:
	/**
	 * Default constructor.
	 */
	StateProcess::StateProcess();

	/**
	 * Destructor.
	 */
	StateProcess::~StateProcess();

	/**
	 * Take status ads, state definition ads, and possibly old state ads,
	 * and generate new state ads.
	 * @param List of status ads.
	 * @param List of state definition ads.
	 * @param List of old state ads (can be empty).
	 * @param List of new state ads (should be empty, ads will be inserted
	 *   into this list).
	 * @return ErrorStatus object giving the status of the operation.
	 */
	ErrorStatus GenerateStateAds(/* const */ ClassAdList &statusAds,
			/* const */ ClassAdList &stateDefAds,
			/* const */ ClassAdList &oldStateAds,
			ClassAdList &newStateAds);

	/**
	 * Get the error status of this object.
	 * @return ErrorStatus of this object.
	 */
	const ErrorStatus &ObjectStatus() const { return objectStatus; }

private:
	/**
	 * Take the status ads and state definition ads and generate a list
	 * of new state ads (not yet combined with the old state ads).
	 * @param List of status ads.
	 * @param List of state definition ads.
	 * @param List of new state ads (should be empty, ads will be inserted
	 *   into this list).
	 * @return ErrorStatus object giving the status of the operation.
	 */
	ErrorStatus NewStateAds(/* const */ ClassAdList &statusAds,
			/* const */ ClassAdList &stateDefAds, ClassAdList &newStateAds);

	/**
	 * Combine the new state ads with the old state ads.
	 * @param List of old state ads (can be empty).
	 * @param List of new state ads (ads will be modified based on the
	 *   old state ads).
	 * @return ErrorStatus object giving the status of the operation.
	 */
	ErrorStatus CombineStateAds(/* const */ ClassAdList &oldStateAds,
			ClassAdList &newStateAds);

	/**
	 * Take a single status ad and state definition ad and put the
	 * corresponding information into the entity state ad.
	 * @param Status ad.
	 * @param State definition ad.
	 * @param The state ad for the entity we're working on (will hold
	 *   info about the state defined in the state definition ad).
	 * @return ErrorStatus object giving the status of the operation.
	 */
	ErrorStatus CheckState(/* const */ ClassAd *status,
			/* const */ClassAd *stateDef,
			ClassAd *stateAd);

	/**
	 * Copy selected attributes from the status ad to the state ad (the
	 * attributes to copy are specified in the CopyAttrs attribute of
	 * the state definition ad).
	 * @param Status ad (input).
	 * @param State definition ad (input).
	 * @param State ad (input/output).
	 */
	ErrorStatus CopyAttrs(/* const */ ClassAd *status,
			/* const */ClassAd *stateDef,
			ClassAd *stateAd);

	/**
	 * Combine old and new states for a given entity, result is kept in
	 * new state ad.
	 * @param The old state ad.
	 * @param The new state ad.
	 * @return ErrorStatus object giving the status of the operation.
	 */
	ErrorStatus CombineEntityStates(/* const */ ClassAd *oldState,
			ClassAd *newState);

	/** 
	 * Determine whether the given tree is a "state sub ad" (a ClassAd
	 * containing a "currentState" attribute).
	 * @param The tree to be tested.
	 * @return True if the tree is a state sub ad, false otherwise.
	 * 
	 */
	bool IsStateSubAd(const ExprTree *tree);

	ErrorStatus	objectStatus;
	ClassAd 	*matchAd;
	ClassAd		*matchStatusAd;
	ClassAd		*matchStateDefAd;
};

#endif /* _StateProcess_h_ */

/*============================================================================*/
