/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-2004 CONDOR Team, Computer Sciences Department,
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.
 * No use of the CONDOR Software Program Source Code is authorized
 * without the express consent of the CONDOR Team.  For more information
 * contact: CONDOR Team, Attention: Professor Miron Livny,
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685,
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure
 * by the U.S. Government is subject to restrictions as set forth in
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison,
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

// Implementation of TriggerUtil class.

#include "trigger_common.h"

#include <iostream>

#include "TriggerUtil.h"
#include "classad_triggers.h"

static const int DEBUG = 0;

//-------------------------------------------------------------------------
ErrorStatus
TriggerUtil::EvalStringOrList(const ClassAd *ad, const string &attrName,
		vector<string *> &values)
{
	DIAG2_OUT("TriggerUtil::EvalStringOrList()");

	ErrorStatus		result;

	Value val;
	if ( ad->EvaluateAttr(attrName, val) ) {
		string stringVal;
		const ExprList *listVal;

		if ( val.IsStringValue(stringVal) ) {
			DIAG2_OUT("DIAG: val is string\n");
			string *tmpStr = new string(stringVal);
			values.push_back(tmpStr);

		} else if ( val.IsListValue(listVal) ) {
			DIAG2_OUT("DIAG: val is list\n");
			EvalState es;
			es.SetScopes(ad);
			for ( ExprList::const_iterator it = listVal->begin();
					it != listVal->end(); it++ ) {
				ExprTree *tree = *it;
				Value subVal;
				if ( tree->Evaluate(es, subVal) ) {
					if ( subVal.IsStringValue(stringVal) ) {
						string *tmpStr = new string(stringVal);
						values.push_back(tmpStr);
					} else {
						ERROR_OUT(attrName + " value is not a string");
						result += ErrorStatus(StatusFailed, ERROR_BAD_CLASSAD);
					}
				} else {
					ERROR_OUT("Unable to evaluate classad attribute: " +
							attrName);
					result += ErrorStatus(StatusFailed, ERROR_BAD_CLASSAD);
				}
			}
		} else {
			ERROR_OUT("\"" + attrName + "\" is not a string or a list");
			result += ErrorStatus(StatusFailed, ERROR_BAD_CLASSAD);
		}
	} else {
		ERROR_OUT("Unable to evaluate attribute " + attrName);
		result += ErrorStatus(StatusFailed, ERROR_BAD_CLASSAD);
	}
	
	return result;
}

//-------------------------------------------------------------------------
ErrorStatus
TriggerUtil::ParseString(const string &input, char delimiter,
		vector<string *> &values)
{
	DIAG2_OUT("TriggerUtil::ParseString(" + input + ", " + delimiter + ")");

	ErrorStatus		result;

	int		pos = 0;
	for ( int index = 0; index < input.size(); ++index ) {
		if ( input[index] == delimiter ) {
			if ( index - pos > 1 ) {
				values.push_back(new string(input, pos, index - pos));
			}
			pos = index + 1;
		}
	}
	if ( input.size() - pos > 1 ) {
		values.push_back(new string(input, pos, input.size() - pos));
	}
	
	return result;
}

//-------------------------------------------------------------------------
ErrorStatus
TriggerUtil::CopyAttr(const ClassAd *inputAd, const string &inputAttr,
		ClassAd *outputAd, const string &outputAttr)
{
	DIAG2_OUT("TriggerUtil::CopyAttr()");

	ErrorStatus		result;

	string		realOutputAttr;
	if ( "" == outputAttr ) {
		realOutputAttr = inputAttr;
	} else {
		realOutputAttr = outputAttr;
	}

	ExprTree *expr = inputAd->Lookup(inputAttr);
	if ( expr ) {
		outputAd->Insert(realOutputAttr, expr->Copy());
	} else {
		ERROR_OUT("Input attribute " + inputAttr + " does not exist");
		result += StatusFailed;
	}

	return result;
}

//-------------------------------------------------------------------------
string
TriggerUtil::SubstringReplace(const string &str, const string &find,
		const string &replace)
{
	DIAG2_OUT("TriggerUtil::SubstringReplace(" + str + ", " + find +
			", " + replace + ")");

	string		result = str;

	int		index = result.find(find);

	if ( index >= 0 ) {
		int		length = find.size();
		result.replace(index, length, replace);
	}

	return result;
}

/*============================================================================*/
