/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-2004 CONDOR Team, Computer Sciences Department,
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.
 * No use of the CONDOR Software Program Source Code is authorized
 * without the express consent of the CONDOR Team.  For more information
 * contact: CONDOR Team, Attention: Professor Miron Livny,
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685,
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure
 * by the U.S. Government is subject to restrictions as set forth in
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison,
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

// Declaration of TriggerUtil class.  This class just contains utility
// functions generally useful in the code.

#ifndef _TriggerUtil_h_
#define _TriggerUtil_h_

#include "ErrorStatus.h"
#include "classad_distribution.h"

class TriggerUtil
{
public:
	/**
	 * Evaluate the given attribute, which should be a string or a list
	 * of strings; put all of the strings into the vector.
	 * Note: the strings are dynamically allocated and should be deleted
	 * by the caller.
	 * @param The ClassAd in which the evaluation is to occur.
	 * @param The name of the attribute to evaluate.
	 * @param The vector to hold the resulting strings.
	 * @return ErrorStatus object giving the status of the operation.
	 */
	static ErrorStatus EvalStringOrList(const ClassAd *ad,
			const string &attrName, vector<string *> &values);

	/**
	 * Parse a string into substrings, using the given delimiter.  Put the
	 * resulting substrings into a vector.
	 * Note: the strings are dynamically allocated and should be deleted
	 * by the caller.
	 * @param The string to parse.
	 * @param The delimiter character.
	 * @param The vector to hold the resulting substrings.
	 * @return ErrorStatus object giving the status of the operation.
	 */
	static ErrorStatus ParseString(const string &input, char delimiter,
			vector<string *> &values);

	/**
	 * Copy an attribute from one ad to another.
	 * @param The ad to copy from.
	 * @param The attribute to copy.
	 * @param The ad to copy to.
	 * @param The attribute to copy to (defaults to the same as the
	 * attribute being copied).
	 * @return ErrorStatus object giving the status of the operation.
	 */
	static ErrorStatus CopyAttr(const ClassAd *inputAd,
			const string &inputAttr, ClassAd *outputAd,
			const string &outputAttr = "");

	/**
	 * Search for the "find" string in the given, replace it with the
	 * "replace" string if found, returning the result (the given
	 * string is unchanged).
	 * @param The string to search.
	 * @param The string to search for.
	 * @param The string to insert.
	 * @param The resulting string.
	 */
	static string SubstringReplace(const string &str, const string &find,
			const string &replace);
};

#endif /* _TriggerUtil_h_ */

/*============================================================================*/
