/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-2004 CONDOR Team, Computer Sciences Department,
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.
 * No use of the CONDOR Software Program Source Code is authorized
 * without the express consent of the CONDOR Team.  For more information
 * contact: CONDOR Team, Attention: Professor Miron Livny,
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685,
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure
 * by the U.S. Government is subject to restrictions as set forth in
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison,
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#include "trigger_common.h"
#include "classad_convert.h"
#include "ClassAdList.h"

//-------------------------------------------------------------------------
int
main(int argc, char **argv)
{
	ErrorStatus status;

	Parameters params(argc, argv);
	status += params.ObjectStatus();

	if (status.IsComplete() && params.run) {
		ClassAdList ads;
		ads.ReadFromFile(params.inputFile,
				params.direction == Parameters::DIR_XML2REG);
		ads.WriteToFile(params.outputFile,
				params.direction == Parameters::DIR_REG2XML);
	}

	return (status.IsComplete() ? 0 : 1);
}

//-------------------------------------------------------------------------
Parameters::Parameters(int argc, char **argv) :
		verbosity(Output::VERBOSITY_DEFAULT),
		direction(DIR_UNKNOWN)
{
	string usage = "Usage: classad_convert -xml2reg|reg2xml [options]\n"
	  "  -in <filename>: file containing input ads (or - to\n"
	  		"    read from stdin)\n"
	  "  -out <filename>: file to receive output ads (or - to\n"
	  		"    write to stdout)\n"
	  "  -usage: print this message and exit\n"
	  "  -verbosity <number>: set verbosity level (default is 3)\n"
	  "  -version: print the version number\n";
	  //TEMP -- change 3 above to Output::VERBOSITY_DEFAULT

	run = true;

	string tmpValue;
	tmpArgc = argc;
	tmpArgv = argv;
	index = 1;
	while ( index < argc && objectStatus.IsComplete() ) {
		if ( getParamWithValue("-in", inputFile) ) {
			// Intentional no-op

		} else if ( getParamWithValue("-out", outputFile) ) {
			// Intentional no-op

		} else if ( getParamWithoutValue("-reg2xml") ) {
			direction = DIR_REG2XML;

		} else if ( getParamWithoutValue("-usage") ) {
			cout << usage;
			run = false;
			return;

		} else if ( getParamWithValue("-verbosity", tmpValue) ) {
			verbosity = (Output::Verbosity)atoi(tmpValue.c_str());
			Output::GetInstance()->SetVerbosity(verbosity);

		} else if ( getParamWithoutValue("-version") ) {
			cout << "classad_convert version " << VERSION << endl;
			run = false;
			return;

		} else if ( getParamWithoutValue("-xml2reg") ) {
			direction = DIR_XML2REG;

		} else {
			ERROR_OUT(string("Illegal argument: ") + string(argv[index]));
			OUTPUT_OUT(usage);

			objectStatus += ErrorStatus(StatusFailed, ERROR_BAD_ARG);
			return;
		}
		++index;
	}

	if ( direction == DIR_UNKNOWN ) {
		ERROR_OUT("Conversion direction must be specified");
		OUTPUT_OUT(usage);

	} else if ( inputFile == "" ) {
		ERROR_OUT("Input file must be specified");
		OUTPUT_OUT(usage);

	} else if ( outputFile == "" ) {
		ERROR_OUT("Output file must be specified");
		OUTPUT_OUT(usage);
	}

	//TEMP -- use macro here?
	if ( verbosity >= Output::VERBOSITY_DIAG1 ) {
		cout << "classad_convert version " << VERSION << endl;
		cout << "Arguments:\n";
		for (int arg = 0; arg < argc; ++arg) {
			cout << "  " << argv[arg] << endl;
		}
	}
}

//-------------------------------------------------------------------------
bool
Parameters::getParamWithoutValue(string paramName) const
{
	bool result = false;
	if ( paramName == string(tmpArgv[index]) ) result = true;
	return result;
}

//-------------------------------------------------------------------------
bool
Parameters::getParamWithValue(string paramName, string &paramValue)
{
	bool result = false;
	if ( getParamWithoutValue(paramName) ) {
		result = true;

		if ( ++index >= tmpArgc ) {
			ERROR_OUT("Parameter " + paramName + " must have a value");
			objectStatus += ErrorStatus(StatusFailed, ERROR_BAD_ARG);
		} else {
			paramValue = tmpArgv[index];
		}
	}
	return result;
}
