/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-2004 CONDOR Team, Computer Sciences Department,
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.
 * No use of the CONDOR Software Program Source Code is authorized
 * without the express consent of the CONDOR Team.  For more information
 * contact: CONDOR Team, Attention: Professor Miron Livny,
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685,
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure
 * by the U.S. Government is subject to restrictions as set forth in
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison,
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#include "trigger_common.h"
#include "classad_triggers.h"
#include "Output.h"
#include "ClassAdList.h"
#include "StateProcess.h"
#include "ProblemProcess.h"
#include "PreprocessStatus.h"

static const int DEBUG = 0;

//-------------------------------------------------------------------------
int
main(int argc, char **argv)
{
	ErrorStatus status;
	CondorErrno = ERR_OK;

	Parameters params(argc, argv);
	status += params.ObjectStatus();

	if (status.IsComplete() && params.run) {

		ClassAdList statusAds;
		if ( status.IsComplete() && params.statusAdsFile != "" ) {
			DIAG1_OUT_NS("Reading status ads...");
			ErrorStatus tmpStatus =
					statusAds.ReadFromFile(params.statusAdsFile, true);
			DIAG1_OUT_NS("...done");
			if ( tmpStatus.IsError() || tmpStatus.IsWarn() ) {
				WARN_OUT("error or warning reading status ads");
			}
			status += tmpStatus;

			if ( status.IsComplete() && params.preProcAdsInFile != "" ) {
				DIAG1_OUT_NS("Reading pre-processing ads...");
				ClassAdList		preProcAds;
				tmpStatus = preProcAds.ReadFromFile(params.preProcAdsInFile,
						false);
				DIAG1_OUT_NS("...done");
				if ( tmpStatus.IsError() || tmpStatus.IsWarn() ) {
					WARN_OUT("error or warning reading pre-processing ads");
				}
				status += tmpStatus;

				if ( status.IsComplete() ) {
					PreprocessStatus ps;
					DIAG1_OUT_NS("Preprocessing status ads...");
					tmpStatus = ps.Process(statusAds, preProcAds);
					DIAG1_OUT_NS("...done");
					if ( tmpStatus.IsError() || tmpStatus.IsWarn() ) {
						WARN_OUT("error or warning preprocessing status ads");
					}
					status += tmpStatus;

					if ( DEBUG >= 3 || params.statusPreFile != "" ) {
						string outFile = params.statusPreFile != "" ?
								params.statusPreFile : "status_pre.ca";
						statusAds.WriteToFile(outFile, false);
					}
				}
			}
		}

		ClassAdList stateDefAds;
		if ( status.IsComplete() && params.stateDefAdsFile != "" ) {
			DIAG1_OUT_NS("Reading state definition ads...");
			ErrorStatus tmpStatus =
					stateDefAds.ReadFromFile(params.stateDefAdsFile, false);
			DIAG1_OUT_NS("...done");
			if ( tmpStatus.IsError() || tmpStatus.IsWarn() ) {
				WARN_OUT("error or warning reading state definition ads");
			}
			status += tmpStatus;
		}

		ClassAdList oldStateAds;
		ClassAdList *stateAds2Process = &oldStateAds;
		if ( status.IsComplete() && params.stateAdsInFile != "" ) {
			DIAG1_OUT_NS("Reading old state ads...");
			ErrorStatus tmpStatus =
					oldStateAds.ReadFromFile(params.stateAdsInFile,
					false).Error2Warning();
			DIAG1_OUT_NS("...done");
			if ( tmpStatus.IsError() || tmpStatus.IsWarn() ) {
				WARN_OUT("error or warning reading old state ads");
			}
			status += tmpStatus;
		}

		if ( statusAds.Size() > 0 ) {
			ClassAdList newStateAds;
			stateAds2Process = &newStateAds;
			if ( status.IsComplete() ) {
				StateProcess sp;
				status += sp.ObjectStatus();
				if ( sp.ObjectStatus().IsComplete() ) {
					DIAG1_OUT_NS("Generating new state ads...");
					ErrorStatus tmpStatus =
							sp.GenerateStateAds(statusAds, stateDefAds,
							oldStateAds, newStateAds);
					DIAG1_OUT_NS("...done");
					if ( tmpStatus.IsError() || tmpStatus.IsWarn() ) {
						WARN_OUT("error or warning generating new state ads");
					}
					status += tmpStatus;
				}
			}

			if ( status.IsComplete() ) {
				DIAG1_OUT_NS("Writing new state ads...");
				ErrorStatus tmpStatus =
						newStateAds.WriteToFile(params.stateAdsOutFile,
						false);
				DIAG1_OUT_NS("...done");
			if ( tmpStatus.IsError() || tmpStatus.IsWarn() ) {
				WARN_OUT("error or warning writing new state ads");
			}
			status += tmpStatus;
			}
		}

		ClassAdList triggerAds;
		if ( status.IsComplete() && params.triggerAdsFile != "" ) {
			DIAG1_OUT_NS("Reading trigger ads...");
			ErrorStatus tmpStatus =
					triggerAds.ReadFromFile(params.triggerAdsFile, false);
			DIAG1_OUT_NS("...done");
			if ( tmpStatus.IsError() || tmpStatus.IsWarn() ) {
				WARN_OUT("error or warning reading trigger ads");
			}
			status += tmpStatus;
		}

		if ( stateAds2Process->Size() > 0 && triggerAds.Size() > 0 ) {
			ProblemProcess pp;
			status += pp.ObjectStatus();
			ClassAdList actionAds;
			if ( status.IsComplete() ) {
				DIAG1_OUT_NS("Generating action ads...");
				ErrorStatus tmpStatus =
						pp.GenerateActionAds(*stateAds2Process, triggerAds,
						actionAds);
				DIAG1_OUT_NS("...done");
				if ( tmpStatus.IsError() || tmpStatus.IsWarn() ) {
					WARN_OUT("error or warning generating action ads");
				}
				status += tmpStatus;
			}

			if ( status.IsComplete() ) {
				DIAG1_OUT_NS("Writing action ads...");
				ErrorStatus tmpStatus =
						actionAds.WriteToFile(params.actionAdsOutFile,
						false);
				DIAG1_OUT_NS("...done");
				if ( tmpStatus.IsError() || tmpStatus.IsWarn() ) {
					WARN_OUT("error or warning writing action ads");
				}
				status += tmpStatus;
			}

			if ( status.IsComplete() ) {
				DIAG1_OUT_NS("Processing action ads...");
				ErrorStatus tmpStatus =
						pp.ProcessActionAds(actionAds);
				DIAG1_OUT_NS("...done");
				if ( tmpStatus.IsError() || tmpStatus.IsWarn() ) {
					WARN_OUT("error or warning processing action ads");
				}
				status += tmpStatus;
			}
		}
	}

	if ( CondorErrno != ERR_OK ) {
		ERROR_OUT("Classad error: " + CondorErrMsg);
		status += StatusFailed;
	}

	if ( DEBUG >= 1 || params.verbosity >= Output::VERBOSITY_DIAG1 ) {
		cout << "Final status: " << status << endl;
	}

	return (status.IsComplete() ? 0 : 1);
}

//-------------------------------------------------------------------------
Parameters::Parameters(int argc, char **argv) :
		verbosity(Output::VERBOSITY_DEFAULT)
{
	string usage = "Usage: condor_trigger [options]\n"
	  "  -actadsout <filename>: file to receive output action ads\n"
	  "  -preprocads <filename>: file containing ads specifying\n"
	  "    status ad pre-procssing\n"
	  "  -shortversion: print the version number only\n"
	  "  -stateadsin <filename>: file containing state ads\n"
	  "  -stateadsout <filename>: file to receive output state ads\n"
	  "  -statedefads <filename>: file containing state definition ads\n"
	  "  -statusads <filename>: file containing status ads (or - to\n"
	  "    read from stdin)\n"
	  "  -statuspreout <filename>: save pre-processed status ads to\n"
	  "    the given file\n"
	  "  -triggerads <filename>: file containing trigger ads\n"
	  "  -usage: print this message and exit\n"
	  "  -verbosity <number>: set verbosity level (default is 3)\n"
	  "  -version: print the version number and compile date\n";
	  //TEMP -- change 3 above to Output::VERBOSITY_DEFAULT

	run = true;

	string tmpValue;
	tmpArgc = argc;
	tmpArgv = argv;
	index = 1;
	while ( index < argc && objectStatus.IsComplete() ) {
		if ( getParamWithValue("-actadsout", actionAdsOutFile) ) {
			// Intentional no-op

		} else if ( getParamWithValue("-preprocads", preProcAdsInFile) ) {
			// Intentional no-op

		} else if ( getParamWithoutValue("-shortversion") ) {
			cout << VERSION << endl;
			run = false;
			return;

		} else if ( getParamWithValue("-stateadsin", stateAdsInFile) ) {
			// Intentional no-op

		} else if ( getParamWithValue("-stateadsout", stateAdsOutFile) ) {
			// Intentional no-op

		} else if ( getParamWithValue("-statedefads", stateDefAdsFile) ) {
			// Intentional no-op

		} else if ( getParamWithValue("-statuspreout", statusPreFile) ) {
			// Intentional no-op

		} else if ( getParamWithValue("-statusads", statusAdsFile) ) {
			// Intentional no-op

		} else if ( getParamWithValue("-triggerads", triggerAdsFile) ) {
			// Intentional no-op

		} else if ( getParamWithoutValue("-usage") ) {
			cout << usage;
			run = false;
			return;

		} else if ( getParamWithValue("-verbosity", tmpValue) ) {
			verbosity = (Output::Verbosity)atoi(tmpValue.c_str());
			Output::GetInstance()->SetVerbosity(verbosity);

		} else if ( getParamWithoutValue("-version") ) {
			cout << "condor_trigger version " << VERSION << " " << __DATE__
					<< endl;
			run = false;
			return;

		} else {
			ERROR_OUT(string("Illegal argument: ") + string(argv[index]));
			OUTPUT_OUT(usage);

			objectStatus += ErrorStatus(StatusFailed, ERROR_BAD_ARG);
			return;
		}
		++index;
	}

	//TEMP -- use macro here?
	if ( verbosity >= Output::VERBOSITY_DIAG1 ) {
		cout << "condor_trigger version " << VERSION << endl;
		cout << "Arguments:\n";
		for (int arg = 0; arg < argc; ++arg) {
			cout << "  " << argv[arg] << endl;
		}
	}
}

//-------------------------------------------------------------------------
bool
Parameters::getParamWithoutValue(string paramName) const
{
	bool result = false;
	if ( paramName == string(tmpArgv[index]) ) result = true;
	return result;
}

//-------------------------------------------------------------------------
bool
Parameters::getParamWithValue(string paramName, string &paramValue)
{
	bool result = false;
	if ( getParamWithoutValue(paramName) ) {
		result = true;

		if ( ++index >= tmpArgc ) {
			ERROR_OUT("Parameter " + paramName + " must have a value");
			objectStatus += ErrorStatus(StatusFailed, ERROR_BAD_ARG);
		} else {
			paramValue = tmpArgv[index];
		}
	}
	return result;
}
